#include "Ftp.h"

Ftp::var_map Ftp::VariableMap;

bool Ftp::initializeVariableMap()
{
	VariableMap["Ftp"] = &Ftp::setExecutable;
	VariableMap["Owner"] = &Ftp::setUser;
	VariableMap["P4FTPPORT"] = &Ftp::setPort;
	VariableMap["P4PORT"] = &Ftp::setTargetPort;
	
	return true;
}

Ftp::Ftp(const std::string& name, const std::string& type, Configuration * config)
	: Executable(name, type, config)
{
}

Ftp::~Ftp()
{
}

void Ftp::changeUser()
{
	if( _user != "root" ) {
		Executable::changeUser();
	}
	// Ftp is allowed to run as root
	// to be able to access port 21
}

void Ftp::verifyVariables() throw(VerificationException)
{
	static bool initialized = initializeVariableMap();
	
	env_map::iterator pos;
	var_map::const_iterator iter;
	
	for (iter = VariableMap.begin(); iter != VariableMap.end(); ++iter) {
		pos = _variables.find(iter->first);
		if (pos != _variables.end()) {
			(this->*iter->second)(pos->second);		
			_variables.erase(pos);		
		}
		else {
			std::string err = _type + " " + _name + ". No " + iter->first + " defined";
			throw VerificationException(err);
		}
	}
}

void Ftp::prepareArgs(arg_vec& args)
{
	std::string procname = _executable + " [" + _port + "]";
	args.push_back(procname);

	args.push_back("-l");
	args.push_back(_port);
	
	addOption("P4LOG", "-L", args);
	addOption("Options", NULL, args);
	
	args.push_back("-p");
	args.push_back(_targetPort);
	
}
