/*******************************************************************************
 *
 * Copyright (c) 2007,  Perforce Software, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#ifndef SERVER_H_
#define SERVER_H_

#include "Executable.h"

class Server : public Executable
{
public:
	typedef void (Server::* var_setter)(const std::string&);
	typedef std::map<std::string, var_setter> var_map;
	
private:
	std::string _root;
	static var_map VariableMap;
	
	static bool initializeVariableMap();
	
public:
	Server(const std::string& name, const std::string& type, Configuration * config);
	virtual ~Server();
	
	void setRoot(const std::string& root) { _root = root; }
	
public:
	virtual void verifyVariables() throw(VerificationException);
	virtual void stopServer();
	virtual void checkpointServer();
	virtual void rotateJournalServer();
	
private:
	void executeCommand(const char * command);
	
	virtual void prepareArgs(arg_vec& args);
};
#endif /*SERVER_H_*/
