#include "Web.h"
#include "assert.h"

Web::var_map Web::VariableMap;

bool Web::initializeVariableMap()
{
	VariableMap["Web"] = &Web::setExecutable;
	VariableMap["Owner"] = &Web::setUser;
	VariableMap["P4WEBPORT"] = &Web::setPort;
	VariableMap["P4PORT"] = &Web::setTargetPort;
	
	return true;	
}

Web::Web(const std::string& name, const std::string& type, Configuration * config)
	: Executable(name, type, config)
{
}

Web::~Web()
{
}

void Web::verifyVariables() throw(VerificationException)
{
	static bool initialized = initializeVariableMap();
	assert(initialized);
	
	env_map::iterator pos;
	var_map::const_iterator iter;
	
	for (iter = VariableMap.begin(); iter != VariableMap.end(); ++iter) {
		pos = _variables.find(iter->first);
		if (pos != _variables.end()) {
			(this->*iter->second)(pos->second);			
			_variables.erase(pos);	
		}
		else {
			std::string err = _type + " " + _name + ". No " + iter->first + " defined";
			throw VerificationException(err);
		}
	}
}
	
void Web::prepareArgs(arg_vec& args)
{
	std::string procname = _executable + " [" + _port + "]";
	args.push_back(procname);

	addOption("Options", NULL, args);
	
	args.push_back("-w");
	args.push_back(_port);

	args.push_back("-p");
	args.push_back(_targetPort);	
}
