#include "configuration.h"

#include <string>
#include <iostream>
#include <algorithm>

using namespace std;

class Exec
{
protected:
	std::string _command;
	bool _executed;
	bool _verbose;
	
public:
	Exec(const std::string& command, bool verbose = false)
		: _command(command), _executed(false), _verbose(verbose)
	{
	}
	
	void operator()(Executable * exec) {
		if (_verbose) {
			std::cout << "Executing : " << *exec << std::endl;
		} 
		exec->executeCommand(_command);
	}	
	
	bool hasExecuted() const { return _executed; }
};

class ExecIfName : public Exec
{
private:
	std::string _name;
public:
	ExecIfName(const std::string& command, const std::string& name, bool verbose = false)
		: Exec(command, verbose), _name(name)
	{
	}
	
	void operator()(Executable * exec) {
		if (exec->getName() == _name) {
			if (_verbose) {
				std::cout << "Executing : " << *exec << std::endl;
			} 
			exec->executeCommand(_command);
			_executed = true;
		}
	} 	
};

class ExecIfType : public Exec
{
private:
	std::string _type;
public:
	ExecIfType(const std::string& command, const std::string& type, bool verbose = false)
		: Exec(command, verbose), _type(type)
	{
	}
	
	void operator()(Executable * exec) {
		if (exec->getType() == _type) {
			if (_verbose) {
				std::cout << "Executing : " << *exec << std::endl;
			} 
			exec->executeCommand(_command);
			_executed = true;
		}
	} 	
};

class ExecIfTypeAndName : public Exec
{
private:
	std::string _type;
	std::string _name;
	
public:
	ExecIfTypeAndName(const std::string& command, const std::string& type, const std::string& name, bool verbose = false)
		: Exec(command, verbose), _type(type), _name(name)
	{
	}
	
	void operator()(Executable * exec) {
		if (exec->getType() == _type && exec->getName() == _name) {
			if (_verbose) {
				std::cout << "Executing : " << *exec << std::endl;
			} 
			exec->executeCommand(_command);
			_executed = true;
		}
	} 	
};


void Configuration::addVariable(const std::string& name, const std::string& value)
{
	_globalEnvironment[name] = value;
}

void Configuration::addExecutable(Executable * executable)
{
	_servers.push_back(executable);
	if (_verbose) {
		std::cout << "found server : " << *executable << std::endl;
	}
}

void Configuration::reverseServerOrder()
{
	reverse(_servers.begin(), _servers.end());
}

bool Configuration::execAll(const std::string& command)
{
	Exec exec(command, _verbose);
	for_each(_servers.begin(), _servers.end(), exec);
	
	return exec.hasExecuted();
}

bool Configuration::execIfName(const std::string& command, const std::string& name)
{
	ExecIfName exec(command, name, _verbose);
	for_each(_servers.begin(), _servers.end(), exec);
	
	return exec.hasExecuted();
}

bool Configuration::execIfType(const std::string& command, const std::string& type)
{
	ExecIfType exec(command, type, _verbose);
	for_each(_servers.begin(), _servers.end(), exec);
	
	return exec.hasExecuted();
}

bool Configuration::execIfTypeAndName(const std::string& command, 
		const std::string& type, const std::string& name)
{
	ExecIfTypeAndName exec(command, type, name, _verbose);
	for_each(_servers.begin(), _servers.end(), exec);
	
	return exec.hasExecuted();
}

