/*******************************************************************************
 *
 * Copyright (c) 2007, Perforce Software, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#ifndef EXCEPTIONS_H_
#define EXCEPTIONS_H_

#include <string>

class ConfigException : public std::exception
{
private:
	std::string _reason;

public:
	ConfigException(const std::string& reason)
	:	_reason(reason)
	{
	}
	
	virtual ~ConfigException() throw() { }
	
	virtual const char* what() const throw()
	{
    	return _reason.c_str();
  	}
  	
};

class VerificationException : public ConfigException
{		
public:
	VerificationException(std::string& reason)
	:	ConfigException(reason)
	{
	}
	
	VerificationException(const char * reason)
	:	ConfigException(std::string(reason))
	{
	}
};

class ParseException : public ConfigException
{
private:
	int _lineno;
	
public:
	ParseException(const std::string& reason, int lineno) 
		: ConfigException(reason), _lineno(lineno)
	{
	}
	
	int lineNumber() const { return _lineno; }
};
#endif /*EXCEPTIONS_H_*/
